"""
    SleekXMPP: The Sleek XMPP Library
    Copyright (C) 2010  Nathanael C. Fritz
    This file is part of SleekXMPP.

    See the file LICENSE for copying permission.
"""


from sleekxmppfs.stanza.error import Error
from sleekxmppfs.stanza.iq import Iq
from sleekxmppfs.stanza.message import Message
from sleekxmppfs.stanza.presence import Presence
from sleekxmppfs.stanza.stream_features import StreamFeatures
from sleekxmppfs.stanza.stream_error import StreamError
