"""
    SleekXMPP: The Sleek XMPP Library
    Copyright (C) 2010 Nathanael C. Fritz, Lance J.T. Stout
    This file is part of SleekXMPP.

    See the file LICENSE for copying permission.
"""

from sleekxmppfs.test.mocksocket import TestSocket
from sleekxmppfs.test.livesocket import TestLiveSocket
from sleekxmppfs.test.sleektest import *
